set_form_variables
set required_variables [list {server_name "Server name"} {server_log "Server log"} {server_error_log "Server Error Log"} {maintainers_email "any email addresses"}]

set error_message ""
foreach required_var $required_variables {
    if {[subst \$[lindex $required_var 0]]==""} {
	append error_message "<li>You failed to enter [lindex $required_var 1].  Please go back and remedy this."
    }
}


set maintainers_email [string trim [tcollins_dectrlM $maintainers_email]]
set maintainers_list [split $maintainers_email \n]
foreach email $maintainers_list {
    if {(![philg_email_valid_p $email])||[regexp { } $email]} {
	append error_message "<li>The email \"$email\" does not seem to be valid."
    }
}

if {![file exists $server_log]} {
    append error_message "<li>The server log you specified ($server_log) does not exist!"
}

if {![file exists $server_error_log]} {
    append error_message "<li>The server error log you specified ($server_error_log) does not exist!"
}

if {$error_message!=""} {
    tcollins_SendError [watchdog_error_page] $error_message
    return
}

#Now the the data checks out, write the new information to our static file
#watchdog-registry.txt
#
#We maintain the information in a list structured so:
#{server_name {maintainers_list} server_log server_error_log {ignore patterns} {{path-patterns {sub-maintainers-email}}...}}

#check to see if a server with this name already exists:
set preexisting [watchdog_access_registry_for_server $server_name]
if {$preexisting!=""} {
    tcollins_SendError [watchdog_error_page] "There is already <a href=\"server-index.tcl?server_name=[ns_urlencode $server_name]\">a server with that name</a>, maintained by [tcollins_mailto [lindex $preexisting 1]]"
    return
}


set this_server_list [list $server_name $maintainers_list $server_log $server_error_log {} {}]

if [catch {watchdog_add_registry_entry $this_server_list} errmsg] {
    tcollins_SendError [watchdog_error_page] "Error while adding to  the registry: $errmsg"
    return
}


#now send confirmation to the user
ReturnHeaders
ns_write "
[watchdog_headers]
<h2>Server $server_name Added</h2>
<p>
You may now <a href=\"server-index.tcl?server_name=[ns_urlencode $server_name]\">administer your watchdog's parameters</a>.
<p>
[watchdog_footers]
"
