ns_share -init {set watchdog_mutex [ns_mutex create]} watchdog_mutex


proc watchdog_headers {} {
    #headers for watchdog html pages
    upvar server_name name
    if [info exists name] {
	return "<html><head><title>Watchdog Maintenance: $name</title></head><body bgcolor=FFFFFF>\n"
    }
    return "<html><head><title>Watchdog Maintenance</title></head><body bgcolor=FFFFFF>\n"
}

proc watchdog_footers {} {
    #footers for watchdog html pages
    return "<hr>\n[tcollins_mailto [watchdog_maintainer_email]]\n</body></html>"
}

proc watchdog_maintainer_email {} {
    #the address of the person maintaining this installation
    return "yourname@yourhost"
}

proc watchdog_page_root {} {
    #where watchdog's files live
    #this should point to the directory which contains
    #the watchdog registry and html files
    return [ns_info pageroot]
}

proc watchdog_error_page {} {
    #the name of watchdog's error page
    return "[watchdog_page_root]/error.html"
}

proc watchdog_add_registry_entry {this_server_list} {
    #how to add entries into the registry file
    ns_share watchdog_mutex
    ns_mutex lock $watchdog_mutex
    ns_log Notice "Appending new server to [watchdog_registry_file]"
    set FILEOUT [open [watchdog_registry_file] a+]
    puts $FILEOUT $this_server_list
    close $FILEOUT
    ns_mutex unlock $watchdog_mutex
    return
}

proc watchdog_registry_file {} {
    #the name of the registry file
    return "[watchdog_page_root]/watchdog-registry.txt"
}

proc watchdog_access_complete_registry {} {
    if {![file exists [watchdog_registry_file]]} {
	return ""
    }
    ns_share watchdog_mutex
    ns_mutex lock $watchdog_mutex
    set FILEIN [open [watchdog_registry_file] r]
    set registry [read -nonewline $FILEIN]
    close $FILEIN
    ns_mutex unlock $watchdog_mutex

    return [split $registry \n]
}

proc watchdog_access_registry_for_server {server_name} {
    #returns a tcl list of the desired server's data
    #or empty string if non-existant
    set all_servers [watchdog_access_complete_registry]

    foreach server $all_servers {
	if {[string compare [lindex $server 0] $server_name]==0} {
	    return $server
	}
    }
    #otherwise return nothing
    return ""
}

proc watchdog_update_registry_for_server {server_name server_data} {
    #we read in the entire registry
    #find the element we want to replace
    #replace it and rewrite the entire file
    #we should be ok thanks to mutex

    set all_servers [watchdog_access_complete_registry]
    if {$all_servers==""} {
	error "watchdog registry ([watchdog_registry_file]) is empty!"
	return
    }

    set desired_index -1
    set current_index 0
    foreach server $all_servers {
	if {[string compare [lindex $server 0] $server_name]==0} {
	    set desired_index $current_index
	    break
	} else {
	    incr current_index
	}
    }

    if {$desired_index==-1} {
	error "$server_name not currently in registry!"
	return
    }

    set all_servers [lreplace $all_servers $desired_index $desired_index $server_data]

    #now we rewrite the file
    ns_share watchdog_mutex
    ns_mutex lock $watchdog_mutex
    ns_log Notice "Updating entry in [watchdog_registry_file]"
    set FILEOUT [open [watchdog_registry_file] w]
    foreach server $all_servers {
	if {$server!=""} {
	    puts $FILEOUT $server
	}
    }
    close $FILEOUT
    ns_mutex unlock $watchdog_mutex

    return
}

proc watchdog_maximum_bytes_to_slurp {} {
    #the maximum amount of bytes to read into memory
    #if the error log is very huge
    #this is 1 meg
    return 1000000
}

proc watchdog_runs_between_summary {} {
    #how many times watchdog runs before a summary is sent
    #we typically schedule watchdog every 15 minutes
    #4 times an hour, 12 hours a day
    return 48
}

