ns_share -init {set watchdog_running 0} watchdog_running

proc watchdog_parse_logs_for_all_servers {} {
    ns_share watchdog_running
    if {$watchdog_running} {
	ns_log Notice "watchdog_parse_logs_for_all_servers: Not running log parse since previous watchdog still running"
	return
    }
    set watchdog_running 1
    
     
    foreach server_data [watchdog_access_complete_registry] {
	ns_log Notice "watchdog_parse_logs_for_all_servers: Running for [lindex $server_data 0]"
	watchdog_parse_logs_for_one_server $server_data
    }
    set watchdog_running 0
}

proc watchdog_parse_logs_for_one_server {server_data} {
    #bytes read from previous runs is stored in global
    #we now slurp the remainder of the file
    #into memory and parse out errors

    set server_name [lindex $server_data 0]
    set server_maintainers [lindex $server_data 1]
    set server_log [lindex $server_data 2]
    set server_error_log [lindex $server_data 3]
    set ignore_patterns [lindex $server_data 4]
    set submaintainers_list [lindex $server_data 5]

    set error_file_marker "${server_name}_error_file_marker"
    set log_file_marker "${server_name}_log_file_marker"
    set watchdog_runs_since_last_summary "${server_name}_runs_since_summary"
    set watchdog_total_errors_since_last_summary "${server_name}_errors_since_summary"

    set shared_variables_to_init [list $error_file_marker $log_file_marker $watchdog_runs_since_last_summary $watchdog_total_errors_since_last_summary]

    foreach shared_variable $shared_variables_to_init {
	ns_share $shared_variable

	if {![info exists $shared_variable]} {
	    set $shared_variable 0
	}
    }

    incr $watchdog_runs_since_last_summary

    #check for new file:
    if [catch {set error_log_size [file size $server_error_log]} errmsg] {
	ns_log Notice "error reading error file: $errmsg"
	foreach server_maintainer $server_maintainers {
	    ns_log Notice "watchdog_parse_logs_for_one_server: sending error notfication to server maintainer $server_maintainer"
	    if [catch {ns_sendmail $server_maintainer [watchdog_maintainer_email] "Unable to parse $server_error_log" "I was unable to parse $server_error_log: $errmsg"} errmsg2] {
		ns_log Notice "watchdog_parse_logs_for_one_server: failed to send email to $server_name maintainer $server_maintainer: $errmsg2"
	    }
	}
	ns_log Notice "moving to next server...."
	return
    }
    if {$error_log_size<[subst \$$error_file_marker]} {
	ns_log Notice "watchdog_parse_logs_for_one_server: since $server_error_log seems to be smaller than last run, reparsing entire error log."
	set $error_file_marker 0
    }

    #begin error_log parsing
    set ERROR_FILEIN [open $server_error_log r]

    #if, since the last reading, the log file has really
    #balooned up, we only want to read in some of the file
    #to memory
    if {[expr $error_log_size - [subst \$$error_file_marker]]>[watchdog_maximum_bytes_to_slurp]} {
	ns_log Notice "watchdog_parse_logs_for_one_server: only reading [watchdog_maximum_bytes_to_slurp] bytes for server $server_name"
	seek $ERROR_FILEIN [expr -1 * [watchdog_maximum_bytes_to_slurp]] end
    } else {
	#set the access position to where we last left off
	seek $ERROR_FILEIN [subst \${$error_file_marker}]
    }

    #either way we're at the eof
    set $error_file_marker $error_log_size

    set errors_found ""

    set error_log_chunk [read $ERROR_FILEIN]
    close $ERROR_FILEIN

    #we still parse line by line
    set error_lines [split $error_log_chunk \n]
    set num_lines [llength $error_lines]
    for {set line_count 0} {$line_count<$num_lines} {incr line_count} {
	set next_line [lindex $error_lines $line_count]
	#first check for errors
	if {[regexp {^(\[[^]]*\])(\[[^]]*\])(\[[^]]*\]) Error: (.*)$}  $next_line match date process server_name_in_logs error_string]} {
	    #now see if we should ignore this error
	    set ignore_error 0
	    foreach pattern $ignore_patterns {
		regsub -all {(\(|\)|\[|\]|\$)} $pattern {\\\1} protected_pattern
		incr ignore_error [regexp "$protected_pattern" $error_string]
		if {$ignore_error} {
		    break;
		}
	    } 
	    
	    if {$ignore_error} {
		continue;
	    }
	    
	    #for "Tcl script failed" errors, we grab the entire
	    #error spouted.  We assume that no other messages
	    #are logged in between the inital error and the raw data
	    if [regexp "Tcl script failed" $error_string] {
		set dated_log_line_count 0
		while {$dated_log_line_count<2} {
		    incr line_count
		    if {$line_count>=$num_lines} {
			break
		    }
		    #otherwise tack the next line on
		    set additional_line [lindex $error_lines $line_count]
		    if {[regexp {^\[[0-9][0-9]/[A-Z][a-z][a-z]/[0-9][0-9][0-9][0-9]:[0-9][0-9]:[0-9][0-9]:[0-9][0-9] [0-9+-][0-9+-][0-9+-][0-9+-][0-9+-]\]\[[0-9.]+\]\[[^]]+\]} $additional_line]} {
			incr dated_log_line_count
		    }
		    if {$dated_log_line_count<2} {
			append next_line "\n$additional_line"
		    } else {
			#this will be the last line, unused
			#we need to roll back because this line
			#might also be an error
			incr line_count -1
		    }
		}
	    } else {
		#some errors are on multiple lines initially
		#so append all lines until we find one that
		#starts with the log date
		set dated_log_line_count 0
		while {$dated_log_line_count<1} {
		    incr line_count
		    if {$line_count>=$num_lines} {
			break
		    }
		    #otherwise tack the next line on
		    set additional_line [lindex $error_lines $line_count]
		    if {[regexp {^\[[0-9][0-9]/[A-Z][a-z][a-z]/[0-9][0-9][0-9][0-9]:[0-9][0-9]:[0-9][0-9]:[0-9][0-9] [0-9+-][0-9+-][0-9+-][0-9+-][0-9+-]\]\[[0-9.]+\]\[[^]]+\]} $additional_line]} {
			incr dated_log_line_count
		    }
		    if {$dated_log_line_count<1} {
			append next_line "\n$additional_line"
		    } else {
			#this will be the last line, unused
			#we need to roll back because this line
			#might also be an error
			incr line_count -1
		    }
		}
	    }
	    #we tack it onto the list of errors
	    lappend errors_found "$next_line"
	    incr $watchdog_total_errors_since_last_summary

	    #see if someone else should know about this
	    foreach submaintainer $submaintainers_list {
		if [regexp "[lindex $submaintainer 0]" $error_string] {
		    lappend errors_for_submaintainers([lindex $submaintainer 1]) "$next_line"
		}
	    }
	}
    }

    #we now have a list of errors for the maintainers
    #and an array of lists of errors for the submaintainers
    ns_log Notice "watchdog_parse_logs_for_one_server: completed parsing log for $server_name, sending necessary email"
    #the maintainer first
    if {[info exists errors_found]&&[llength $errors_found]>0} {
	set maintainer_body "Runs since last summary: [subst \$$watchdog_runs_since_last_summary]\nRuns betweeen summaries: [watchdog_runs_between_summary]\n\nWatchdog has found the following [llength $errors_found] errors since it last remembers running:\n\n"
	foreach error_found $errors_found {
	    append maintainer_body "---------------\n$error_found\n\n"
	}
	foreach server_maintainer $server_maintainers {
	    ns_log Notice "watchdog_parse_logs_for_one_server: sending email to server maintainer $server_maintainer"
	    if [catch {ns_sendmail $server_maintainer [watchdog_maintainer_email] "Errors found in $server_error_log" "$maintainer_body"} errmsg] {
		ns_log Notice "watchdog_parse_logs_for_one_server: failed to send email to $server_name maintainer $server_maintainer: $errmsg"
	    }
	}
    }
    #now the submaintainers
    if [info exists errors_for_submaintainers] {
	foreach submaintainer [array names errors_for_submaintainers] {
	    set submaintainer_body "Watchdog has found the following [llength $errors_for_submaintainers($submaintainer)] errors since it last remembers running:\n\n"
	    foreach error_found $errors_for_submaintainers($submaintainer) {
		append submaintainer_body "-----------------\n$error_found\n\n"
	    }
	    ns_log Notice "watchdog_parse_logs_for_one_server: sending email to server submaintainer $submaintainer"
	    if [catch {ns_sendmail $submaintainer [watchdog_maintainer_email] "Errors found in $server_error_log" "$submaintainer_body"} errmsg] {
		ns_log Notice "watchdog_parse_logs_for_one_server: failed to send email to $server_name sub-maintainer $submaintainer: $errmsg"
	    }
	}
    }

    ns_log Notice "run #[subst \$$watchdog_runs_since_last_summary] out of [watchdog_runs_between_summary]"

    #check to see if it's time to mail out a summary info
    if {[subst \$$watchdog_runs_since_last_summary]>=[watchdog_runs_between_summary]} {
	set summary_email "Watchdog has completed [subst \$$watchdog_runs_since_last_summary] runs since the last summary, and found [subst \$$watchdog_total_errors_since_last_summary] errors since then.  These counts are now being reset."
	foreach server_maintainer $server_maintainers {
	    ns_log Notice "watchdog_parse_logs_for_one_server: sending summary email to server maintainer $server_maintainer"
	    if [catch {ns_sendmail $server_maintainer [watchdog_maintainer_email] "Watchdog Summary" "$summary_email"} errmsg] {
		ns_log Notice "watchdog_parse_logs_for_one_server: failed to send summary email to $server_name maintainer $server_maintainer: $errmsg"
	    }
	}
	set $watchdog_total_errors_since_last_summary 0
	set $watchdog_runs_since_last_summary 0
    }

    #we are done with this run
    return
}

ns_share -init {set watchdog_procs_scheduled_p 0} watchdog_procs_scheduled_p

if { !$watchdog_procs_scheduled_p } {
    set watchdog_procs_scheduled_p 1
    ns_schedule_proc -thread [expr 15 * 60] watchdog_parse_logs_for_all_servers
}
